#include "grn.h"

int ExpressionData::comp(string s1, string s2) { // 
  if (s1.size() != s2.size())
    return FALSE;
  for (int i = 0; i < s1.size(); i++) 
    if (toupper(s1.c_str()[i]) != toupper(s2.c_str()[i]))
      return FALSE;
  return TRUE;
}

int ExpressionData::inNames(string g) {
  for (int i = 0; i < numberOfGenes; i++)
    if (comp(orf[i].name, g) == TRUE)
      return i;
  return -1;
}

ExpressionData::ExpressionData(float ml, float wl) { 
  orf = new ORF[NOG];
  numberOfGenes = 0;
  namesFile  = "data/orfNames.txt";
  rrFile     = "data/reg2814.txt"; // "art1018.txt"; //art1219.txt"; // "in1/art.txt";
  expFile[0] = "data/cdc15.txt";
  noe[0]     = 24;
  expFile[1] = "data/alpha.txt";
  noe[1]     = 18;
  expFile[2] = "data/cdc28.txt";
  noe[2]     = 17;
  expFile[3] = "data/elu.txt";
  noe[3]     = 14;

  rr[0] = new Rr[NOREG];
  rr[1] = new Rr[NOREG];
  rr[2] = new Rr[NOREG];
  rr[3] = new Rr[NORR];

  midL  = ml;
  widL  = wl;

  readExpData();
  normolizExpData();
  // quality();
  readRR();
  // cerr << " READ_RR is done" << endl; 
  discritize();
  // cerr << " discritize() is done" << endl; 
  // display(0);
}

void ExpressionData::readExpData() {
  ifstream fin, fdin[NOF];
  string   name;
  int      e;
  int      nom;
  char     ch;
  
  fin.open(namesFile.c_str());
  while (TRUE) {
    fin >> name;
    if (fin.eof())
      break;
    orf[numberOfGenes].name = name;
    numberOfGenes++;
  }
  fin.close();
  
  for (int f = 0; f < NOF; f++) {
    fdin[f].open(expFile[f].c_str());
    for (int g = 0; g < numberOfGenes; g++) {
      e   = 0; 
      nom = 0;
      while(TRUE) { // for (e = 0; e < noe[f]; e++) {
	while ((ch = fdin[f].peek()) == ' ')
	  (fdin[f]).get(ch);  
	if (ch == 10 || ch == 13) { // 10 is LF, 13 is CR
	  fdin[f].get(ch);
	  if (ch == 10) {
	    ch = fdin[f].peek();
	    if (ch == 13) {
	      fdin[f].get(ch);
	    }
	  } else {
	    ch = fdin[f].peek();
	    if (ch == 10) {
	      fdin[f].get(ch);
	    }
	  }
	  break;
	}
	if (isdigit(ch) || ch == '-') { // read value
	  fdin[f] >> orf[g].eLevel[f][e];
	  orf[g].valid[f][e] = TRUE;
	  if (orf[g].eLevel[f][e] > orf[g].max[f]) {
	    orf[g].max[f] = orf[g].eLevel[f][e];
	    orf[g].maxIDX[f] = e;
	  }
	  if (orf[g].eLevel[f][e] < orf[g].min[f]) {
	    orf[g].min[f] = orf[g].eLevel[f][e];
	    orf[g].minIDX[f] = e;
	  }
	} else if (ch == 9) { // 9 is tab
	  fdin[f].get(ch);
	  e++;
	} 
      }
      orf[g].noe[f] = noe[f];
    }
    fdin[f].close();
  }
} // readExpData

void ExpressionData::readRR() {
  ifstream fin;
  string   reg;
  string   tar;
  char     type;
  int      regID;
  int      tarID;
  int      t;
  
  srand((unsigned) time(0));
  rrSZ[RR] = rrSZ[AA] = 0;
  fin.open(rrFile.c_str());
  while (TRUE) {
    fin >> reg;
    if (fin.eof())
      break;
    fin >> type;
    fin >> tar;
    tarID = inNames(tar);
    regID = inNames(reg);

    if (type == 'A') {
      t = AA;
    } else {
      t = RR;
    }
    if (regID == -1) {
      // cerr << "ErrorreadRR (reg):\t" << reg << "\t" << reg << "\t" << type << "\t" << tar << endl;
    } else if (tarID == -1) {
      // cerr << "ErrorreadRR (tar):\t" << tar << "\t" << reg << "\t" << type << "\t" << tar << endl;
    } else {
      rr[t][rrSZ[t]].reg  = regID;
      rr[t][rrSZ[t]].tar  = tarID;
      rr[t][rrSZ[t]].type = type;
      rrSZ[t]++;
    }
  }
  fin.close();
  for (int i = 0; i < NOREG; i++) {
    rr[2][i].reg = 0;
    rr[2][i].tar = 0;
    rr[2][i].type = 'A';
    while(int(rr[2][i].tar) == int(rr[2][i].reg)) {
      rr[2][i].reg = int(rnd(6000));
      rr[2][i].tar = int(rnd(6000));
    }
  }
  rrSZ[2] = NOREG;
  for (int i = 0; i < NORR; i++) {
    rr[3][i].reg = 0;
    rr[3][i].tar = 0;
    rr[3][i].type = 'A';
    while(int(rr[3][i].tar) == int(rr[3][i].reg)) {
      rr[3][i].reg = int(rnd(6000));
      rr[3][i].tar = int(rnd(6000));
    }
  }
  rrSZ[3] = NORR;
}

void ExpressionData::normolizExpData() {
  float wid;
  for (int f = 0; f < NOF; f++) {
    for (int g = 0; g < numberOfGenes; g++) {
      wid = orf[g].max[f] - orf[g].min[f];
      for (int e = 0; e < noe[f]; e++) {
	if (orf[g].valid[f][e] == TRUE) {
	  orf[g].eLevel[f][e] = (orf[g].eLevel[f][e] - orf[g].min[f]) / wid;
	}
      }
    }
  }
}

void ExpressionData::quality() {
  int lmx;
  int lmn;
  int ltot;
  int vld;
  for (int f = 0; f < NOF; f++) {
    lmx = lmn = ltot = 0;
    cout << expFile[f] << "\t";
    for (int g = 0; g < numberOfGenes; g++) {
      for (int e = 1; e < (noe[f] - 1); e++) {
	if (orf[g].valid[f][e - 1] == TRUE && 
	    orf[g].valid[f][e] == TRUE && 
	    orf[g].valid[f][e + 1] == TRUE) {
	  ltot++;
	  if (orf[g].eLevel[f][e] < orf[g].eLevel[f][e - 1] && 
	      orf[g].eLevel[f][e] < orf[g].eLevel[f][e + 1])
	    lmn++;
	  if (orf[g].eLevel[f][e] > orf[g].eLevel[f][e - 1] && 
	      orf[g].eLevel[f][e] > orf[g].eLevel[f][e + 1])
	    lmx++;
	}
      }
    }
    cout << int(100*float(lmx + lmn)/float(ltot))/100.0 << "\t" 
	 << int(100*float(lmx)/float(ltot))/100.0 << "\t" 
	 << int(100*float(lmn)/float(ltot))/100.0 << endl;
  }
}

void ExpressionData::discritize() {
  for (int f = 0; f < NOF; f++) {
    for (int g = 0; g < numberOfGenes; g++) {
      for (int e = 0; e < noe[f]; e++) {
	if (orf[g].valid[f][e] == TRUE) {
	  if (orf[g].eLevel[f][e] < (midL -widL)) 
	    orf[g].catLev[f][e] = -1;
	  else if (orf[g].eLevel[f][e] > (midL + widL))
	    orf[g].catLev[f][e] = 1;
	  else
	    orf[g].catLev[f][e] = 0;
	}
      }
    }
  }
} 

void ExpressionData::displayRR() {
  ofstream fout;
  fout.open("out3EP.txt");
  fout << "\nrE#\tReg\tType\tTarg\t";
  for (int z = SLEN; z < MAXSQSZ; z++)
    fout << z << '\t';
  fout << endl;
  for (int tt = AA; tt <= RR; tt++) {
    for (int r = 0; r < rrSZ[tt]; r++) {
      fout << r << "\t" << rr[tt][r].reg << '\t' << rr[tt][r].type << "\t" 
	   << rr[tt][r].tar << '\t';
      for (int z = SLEN; z < MAXSQSZ; z++)
	fout << rr[tt][r].eCount[z] << '\t';
      fout << endl;
    }
  }

  fout << "\nrV#\tReg\tType\tTarg\t";
  for (int z = SLEN; z < MAXSQSZ; z++)
    fout << z << '\t';
  fout << endl;
  for (int tt = AA; tt <= RR; tt++) {
    for (int r = 0; r < rrSZ[tt]; r++) {
      fout << r << "\t" << rr[tt][r].reg << '\t' << rr[tt][r].type << "\t" 
	   << rr[tt][r].tar << '\t';
      // for (int z = SLEN; z < MAXSQSZ; z++)
      //   fout << rr[tt][r].vCount[z] << '\t';
      fout << endl;
    }
  }

  fout << "\nrEF#\tReg\tType\tTarg\tFile\t";
  for (int z = SLEN; z < MAXSQSZ; z++)
    fout << z << '\t';
  fout << endl;
  for (int tt = AA; tt <= RR; tt++) {
    for (int r = 0; r < rrSZ[tt]; r++) {
      for (int f = 0; f < NOF; f++) {
	fout << r << "\t" << rr[tt][r].reg << '\t' << rr[tt][r].type << "\t" 
	     << rr[tt][r].tar 
	     << '\t' << f << '\t';
	for (int z = SLEN; z < MAXSQSZ; z++)
	  fout << rr[tt][r].eCountF[z][f] << '\t';
	fout << endl;
      }
    }
  }

  fout << "\nrVF#\tReg\tType\tTarg\tFile\t";
  for (int z = SLEN; z < MAXSQSZ; z++)
    fout << z << '\t';
  fout << endl;
  for (int tt = AA; tt <= RR; tt++) {
    for (int r = 0; r < rrSZ[tt]; r++) {
      for (int f = 0; f < NOF; f++) {
	fout << r << "\t" << rr[tt][r].reg << '\t' << rr[tt][r].type << "\t" 
	     << rr[tt][r].tar
	     << '\t' << f << '\t';
	// for (int z = SLEN; z < MAXSQSZ; z++)
	//   fout << rr[tt][r].vCountF[z][f] << '\t';
	fout << endl;
      }
    }
  }

  for (int tt = AA; tt <= RR; tt++) {
    fout << " -- Type:\t" << tt << endl;
    for (int r = 0; r < rrSZ[tt]; r++) {
      fout << r << ")\tReg:\t" << rr[tt][r].reg << '\t' << rr[tt][r].type 
	   << "\tTar:\t" << rr[tt][r].tar << endl;
      for (int f = 0; f < NOF; f++) {
	fout << "\nfile:\t" << f << "\tSeq Sz:\t" << noe[f] << "\tSeq:";
	for (int e = 0; e < noe[f]; e++) {
	  if (rr[tt][r].vld[f][e] == TRUE)
	    fout << '\t' << rr[tt][r].seq[f][e];
	  else
	    fout << "\tX";
	}
	fout << endl;
	fout << "\tNum of Ep:\t" << rr[tt][r].noep[f] << endl;
	for (int p = 0; p < rr[tt][r].noep[f]; p++) {
	  fout << "EP" << p << "\tEpSize:\t" << rr[tt][r].epSize[f][p] << "\tEpStart:\t" 
	       << rr[tt][r].epStart[f][p] << "\tEpEnd:\t" << rr[tt][r].epEnd[f][p] << "\tEp:\t";
	  for (int e = 0; e < rr[tt][r].epSize[f][p]; e++) {
	    fout << rr[tt][r].ep[f][p][e] << '\t';
	  }
	  fout << endl;
	}
	/* 
	fout << "\tNum of Rv:\t" << rr[tt][r].norv[f] << endl;
	for (int v = 0; v < rr[tt][r].norv[f]; v++) {
	  fout << "RV" << v << "\tRvSize:\t" << rr[tt][r].rvSize[f][v] << "\tRvStart:\t" 
	       << rr[tt][r].rvStart[f][v] << "\tRvEnd:\t" << rr[tt][r].rvEnd[f][v] << "\tRv:\t";
	  for (int e = 0; e < rr[tt][r].rvSize[f][v]; e++) {
	    fout << rr[tt][r].rv[f][v][e] << '\t';
	  }
	  fout << endl;
	}
	*/
      }
      fout << endl;
    } // r
  }
  fout.close();
}

/*
void ExpressionData::displayRRScores(int r, int tt) {
  fgout << r << "\t" << orf[rr[tt][r].reg].name << '\t' << rr[tt][r].type << "\t" 
       << orf[rr[tt][r].tar].name << "\t:\t";
  for (int i = 0; i < 8; i++) {
    fgout << rr[tt][r].scoreAvr[i] << '\t';
  }
  fgout << endl;
}
*/
float ExpressionData::rnd(float mx) {
  return rnd() * mx;
}

float ExpressionData::rnd() {
  float rnd;
  float mx;
  mx = RAND_MAX;
  rnd = rand();
  return rnd / mx;
}

